public class PrimeTester {
    private final int MAX;
    private final int ROOT_MAX;
    private final int[] SMALL_PRIMES;

    public PrimeTester(int maxValue) {
	MAX = maxValue;
	ROOT_MAX = (int) Math.sqrt(MAX);
	SMALL_PRIMES = getSmallPrimes();
	
    }

    public boolean isPrime(int n) {
	if (n < 0 || n > MAX) {
	    throw new IndexOutOfBoundsException();
	}

	if (n == 0 || n == 1) {
	    return false;
	}

	for (int p : SMALL_PRIMES) {
	    if (n != p && n % p == 0) {
		return false;
	    }
	}

	return true;
    }

    public int[] getSmallPrimes() {
	boolean[] isPrime = new boolean[ROOT_MAX];

	for (int i = 0; i < ROOT_MAX; ++i) {
	    isPrime[i] = true;
	}

	isPrime[0] = false;
	isPrime[1] = false;

	for (int i = 2; i < ROOT_MAX; ++i) {
	    int rootSize = (int) Math.sqrt(ROOT_MAX);
	    if (isPrime[i]) {
		if (i <= rootSize) {
		    int j = i * i;
		    while (j < ROOT_MAX) {
			isPrime[j] = false;
			j += i;
		    }
		}
	    }
	}

	int primeCount = 0;

	for (boolean b : isPrime) {
	    if (b)
		primeCount++;
	}

	int[] primes = new int[primeCount];
	int pIndex = 0;

	for (int i = 0; i < ROOT_MAX; ++i) {
	    if (isPrime[i]) {
		primes[pIndex] = i;
		++pIndex;
	    }
	}

	return primes;
    }

}
